package web

import (
	"context"
	"log"
	"time"

	"git.loafle.net/commons_go/rpc/gateway"
	"git.loafle.net/commons_go/server"
	cwf "git.loafle.net/commons_go/websocket_fasthttp"
	"git.loafle.net/overflow/overflow_gateway_app/conf"
	"github.com/valyala/fasthttp"
)

func NewSocketHandler() (entry string, socketHandler cwf.SocketHandler) {
	sh := &SocketHandlers{}

	cfg := conf.Config.Handlers["web"]

	sh.entryPath = cfg.Entry

	sh.MaxMessageSize = cfg.Socket.MaxMessageSize
	sh.WriteTimeout = cfg.Socket.WriteTimeout * time.Second
	sh.ReadTimeout = cfg.Socket.ReadTimeout * time.Second
	sh.PongTimeout = cfg.Socket.PongTimeout * time.Second
	sh.PingTimeout = cfg.Socket.PingTimeout * time.Second
	sh.PingPeriod = cfg.Socket.PingPeriod * time.Second

	sh.rpcServerHandler = newRCPServerHandler()

	return sh.entryPath, sh
}

type SocketHandlers struct {
	cwf.SocketHandlers

	entryPath string

	rpcServerHandler gateway.ServerHandler
}

func (sh *SocketHandlers) Handshake(ctx *fasthttp.RequestCtx) (interface{}, *fasthttp.ResponseHeader) {
	return "overflow@loafle.com", nil
}

func (sh *SocketHandlers) OnConnect(soc *cwf.Socket) {
	socketManager.putSocket(soc.ID().(string), soc)
}

func (sh *SocketHandlers) Handle(soc *cwf.Socket, stopChan <-chan struct{}, doneChan chan<- struct{}) {
	contentType := "json"
	codec, err := sh.rpcServerHandler.GetCodec(contentType)
	if nil != err {
		log.Printf("RPC Handle: %v", err)
		doneChan <- struct{}{}
		return
	}

	var socConn *cwf.SocketConn
	baseCtx := context.Background()
	// conn.SetPongHandler(func(string) error { c.conn.SetReadDeadline(time.Now().Add(pongWait)); return nil })

	for {
		if socConn, err = soc.WaitRequest(); nil != err {
			doneChan <- struct{}{}
			return
		}

		// // "git.loafle.net/commons_go/websocket_fasthttp/websocket"
		// switch socConn.MessageType {
		// case websocket.TextMessage:
		// case websocket.BinaryMessage:
		// }
		ctx := context.WithValue(baseCtx, "socket", soc)

		if err = gateway.Handle(ctx, sh.rpcServerHandler, codec, socConn, socConn); nil != err {
			if server.IsClientDisconnect(err) {
				doneChan <- struct{}{}
				return
			}
			log.Printf("RPC: %v", err)
		}

		if err = socConn.Close(); nil != err {
			doneChan <- struct{}{}
			return
		}

		select {
		case <-stopChan:
			return
		default:
		}
	}

}

func (sh *SocketHandlers) OnDisconnect(soc *cwf.Socket) {
	socketManager.removeSocket(soc.ID().(string))
}

func (sh *SocketHandlers) Validate() {
	sh.SocketHandlers.Validate()

	if nil == sh.rpcServerHandler {
		panic("RPCServerHandler must be specified.")
	}
}

var socketManager webSocketManager

type webSocketManager struct {
	sockets map[string]*cwf.Socket
}

func (sm *webSocketManager) putSocket(id string, soc *cwf.Socket) {
	if nil == sm.sockets {
		sm.sockets = make(map[string]*cwf.Socket)
	}
	sm.sockets[id] = soc
}

func (sm *webSocketManager) removeSocket(id string) {
	delete(sm.sockets, id)
}

func (sm *webSocketManager) getSocket(id string) *cwf.Socket {
	if nil == sm.sockets {
		return nil
	}
	return sm.sockets[id]
}
